import { localize } from '../../nls.js';
const minute = 60;
const hour = minute * 60;
const day = hour * 24;
const week = day * 7;
const month = day * 30;
const year = day * 365;
function fromNow(date, appendAgoLabel, useFullTimeWords, disallowNow) {
    if (typeof date !== 'number') {
        date = date.getTime();
    }
    const seconds = Math.round((( (new Date())).getTime() - date) / 1000);
    if (seconds < -30) {
        return ( localize(
            838,
            'in {0}',
            fromNow(( (new Date())).getTime() + seconds * 1000, false)
        ));
    }
    if (!disallowNow && seconds < 30) {
        return ( localize(839, 'now'));
    }
    let value;
    if (seconds < minute) {
        value = seconds;
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(840, '{0} second ago', value))
                    : ( localize(841, '{0} sec ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(842, '{0} seconds ago', value))
                    : ( localize(843, '{0} secs ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(844, '{0} second', value))
                    : ( localize(845, '{0} sec', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(846, '{0} seconds', value))
                    : ( localize(847, '{0} secs', value));
            }
        }
    }
    if (seconds < hour) {
        value = Math.floor(seconds / minute);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(848, '{0} minute ago', value))
                    : ( localize(849, '{0} min ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(850, '{0} minutes ago', value))
                    : ( localize(851, '{0} mins ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(852, '{0} minute', value))
                    : ( localize(853, '{0} min', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(854, '{0} minutes', value))
                    : ( localize(855, '{0} mins', value));
            }
        }
    }
    if (seconds < day) {
        value = Math.floor(seconds / hour);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(856, '{0} hour ago', value))
                    : ( localize(857, '{0} hr ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(858, '{0} hours ago', value))
                    : ( localize(859, '{0} hrs ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(860, '{0} hour', value))
                    : ( localize(861, '{0} hr', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(862, '{0} hours', value))
                    : ( localize(863, '{0} hrs', value));
            }
        }
    }
    if (seconds < week) {
        value = Math.floor(seconds / day);
        if (appendAgoLabel) {
            return value === 1
                ? ( localize(864, '{0} day ago', value))
                : ( localize(865, '{0} days ago', value));
        }
        else {
            return value === 1
                ? ( localize(866, '{0} day', value))
                : ( localize(867, '{0} days', value));
        }
    }
    if (seconds < month) {
        value = Math.floor(seconds / week);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(868, '{0} week ago', value))
                    : ( localize(869, '{0} wk ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(870, '{0} weeks ago', value))
                    : ( localize(871, '{0} wks ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(872, '{0} week', value))
                    : ( localize(873, '{0} wk', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(874, '{0} weeks', value))
                    : ( localize(875, '{0} wks', value));
            }
        }
    }
    if (seconds < year) {
        value = Math.floor(seconds / month);
        if (appendAgoLabel) {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(876, '{0} month ago', value))
                    : ( localize(877, '{0} mo ago', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(878, '{0} months ago', value))
                    : ( localize(879, '{0} mos ago', value));
            }
        }
        else {
            if (value === 1) {
                return useFullTimeWords
                    ? ( localize(880, '{0} month', value))
                    : ( localize(881, '{0} mo', value));
            }
            else {
                return useFullTimeWords
                    ? ( localize(882, '{0} months', value))
                    : ( localize(883, '{0} mos', value));
            }
        }
    }
    value = Math.floor(seconds / year);
    if (appendAgoLabel) {
        if (value === 1) {
            return useFullTimeWords
                ? ( localize(884, '{0} year ago', value))
                : ( localize(885, '{0} yr ago', value));
        }
        else {
            return useFullTimeWords
                ? ( localize(886, '{0} years ago', value))
                : ( localize(887, '{0} yrs ago', value));
        }
    }
    else {
        if (value === 1) {
            return useFullTimeWords
                ? ( localize(888, '{0} year', value))
                : ( localize(889, '{0} yr', value));
        }
        else {
            return useFullTimeWords
                ? ( localize(890, '{0} years', value))
                : ( localize(891, '{0} yrs', value));
        }
    }
}
function fromNowByDay(date, appendAgoLabel, useFullTimeWords) {
    if (typeof date !== 'number') {
        date = date.getTime();
    }
    const todayMidnightTime = ( (new Date()));
    todayMidnightTime.setHours(0, 0, 0, 0);
    const yesterdayMidnightTime = ( (new Date(todayMidnightTime.getTime())));
    yesterdayMidnightTime.setDate(yesterdayMidnightTime.getDate() - 1);
    if (date > todayMidnightTime.getTime()) {
        return ( localize(892, 'Today'));
    }
    if (date > yesterdayMidnightTime.getTime()) {
        return ( localize(893, 'Yesterday'));
    }
    return fromNow(date, appendAgoLabel, useFullTimeWords);
}
function getDurationString(ms, useFullTimeWords) {
    const seconds = Math.abs(ms / 1000);
    if (seconds < 1) {
        return ( localize(895, '{0}ms', ms));
    }
    if (seconds < minute) {
        return ( localize(897, '{0}s', Math.round(ms) / 1000));
    }
    if (seconds < hour) {
        return ( localize(899, '{0} mins', Math.round(ms / (1000 * minute))));
    }
    if (seconds < day) {
        return ( localize(901, '{0} hrs', Math.round(ms / (1000 * hour))));
    }
    return ( localize(902, '{0} days', Math.round(ms / (1000 * day))));
}
function toLocalISOString(date) {
    return date.getFullYear() +
        '-' + String(date.getMonth() + 1).padStart(2, '0') +
        '-' + String(date.getDate()).padStart(2, '0') +
        'T' + String(date.getHours()).padStart(2, '0') +
        ':' + String(date.getMinutes()).padStart(2, '0') +
        ':' + String(date.getSeconds()).padStart(2, '0') +
        '.' + (date.getMilliseconds() / 1000).toFixed(3).slice(2, 5) +
        'Z';
}
export { fromNow, fromNowByDay, getDurationString, toLocalISOString };
