import { __decorate, __param } from '../../../../../../external/tslib/tslib.es6.js';
import { localize } from '../../../nls.js';
import { getActiveWindow, addDisposableListener } from '../../../base/browser/dom.js';
import { createFastDomNode } from '../../../base/browser/fastDomNode.js';
import { BugIndicatingError } from '../../../base/common/errors.js';
import { Disposable } from '../../../base/common/lifecycle.js';
import { observableValue } from '../../../base/common/observableInternal/base.js';
import '../../../base/common/arrays.js';
import '../../../base/common/event.js';
import '../../../base/common/observableInternal/autorun.js';
import '../../../base/common/observableInternal/derived.js';
import '../../../base/common/cancellation.js';
import { runOnChange } from '../../../base/common/observableInternal/utils.js';
import { IInstantiationService } from '../../../platform/instantiation/common/instantiation.js';
import { TextureAtlas } from './atlas/textureAtlas.js';
import { IConfigurationService } from '../../../platform/configuration/common/configuration.service.js';
import '../../../platform/notification/common/notification.js';
import { INotificationService } from '../../../platform/notification/common/notification.service.js';
import { GPULifecycle } from './gpuDisposable.js';
import { ensureNonNullable, observeDevicePixelDimensions } from './gpuUtils.js';
import { RectangleRenderer } from './rectangleRenderer.js';
import Severity$1 from '../../../base/common/severity.js';
var ViewGpuContext_1;
let ViewGpuContext = class ViewGpuContext extends Disposable {
    static { ViewGpuContext_1 = this; }
    static get atlas() {
        if (!ViewGpuContext_1._atlas) {
            throw ( (new BugIndicatingError('Cannot call ViewGpuContext.textureAtlas before device is resolved')));
        }
        return ViewGpuContext_1._atlas;
    }
    get atlas() {
        return ViewGpuContext_1.atlas;
    }
    constructor(context, _instantiationService, _notificationService, configurationService) {
        super();
        this._instantiationService = _instantiationService;
        this._notificationService = _notificationService;
        this.configurationService = configurationService;
        this.canvas = createFastDomNode(document.createElement('canvas'));
        this.canvas.setClassName('editorCanvas');
        this.ctx = ensureNonNullable(this.canvas.domNode.getContext('webgpu'));
        this.device = GPULifecycle.requestDevice((message) => {
            const choices = [{
                    label: ( localize(5133, "Use DOM-based rendering")),
                    run: () => this.configurationService.updateValue('editor.experimentalGpuAcceleration', 'off'),
                }];
            this._notificationService.prompt(Severity$1.Warning, message, choices);
        }).then(ref => this._register(ref).object);
        this.device.then(device => {
            if (!ViewGpuContext_1._atlas) {
                ViewGpuContext_1._atlas = this._instantiationService.createInstance(TextureAtlas, device.limits.maxTextureDimension2D, undefined);
                runOnChange(this.devicePixelRatio, () => ViewGpuContext_1.atlas.clear());
            }
        });
        this.rectangleRenderer = this._instantiationService.createInstance(RectangleRenderer, context, this.canvas.domNode, this.ctx, this.device);
        const dprObs = observableValue(this, getActiveWindow().devicePixelRatio);
        this._register(addDisposableListener(getActiveWindow(), 'resize', () => {
            dprObs.set(getActiveWindow().devicePixelRatio, undefined);
        }));
        this.devicePixelRatio = dprObs;
        const canvasDevicePixelDimensions = observableValue(this, { width: this.canvas.domNode.width, height: this.canvas.domNode.height });
        this._register(observeDevicePixelDimensions(this.canvas.domNode, getActiveWindow(), (width, height) => {
            this.canvas.domNode.width = width;
            this.canvas.domNode.height = height;
            canvasDevicePixelDimensions.set({ width, height }, undefined);
        }));
        this.canvasDevicePixelDimensions = canvasDevicePixelDimensions;
    }
    static canRender(options, viewportData, lineNumber) {
        const data = viewportData.getViewLineRenderingData(lineNumber);
        if (data.containsRTL ||
            data.maxColumn > 200 ||
            data.continuesWithWrappedLine ||
            data.inlineDecorations.length > 0) {
            return false;
        }
        return true;
    }
};
ViewGpuContext = ViewGpuContext_1 = ( (__decorate([
    ( (__param(1, IInstantiationService))),
    ( (__param(2, INotificationService))),
    ( (__param(3, IConfigurationService)))
], ViewGpuContext)));
export { ViewGpuContext };
