import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { marked } from 'marked';
import { localize } from '../../../../nls.js';
import { IAccessibleViewService } from '../../../../platform/accessibility/browser/accessibleView.service.js';
import { isRequestVM, isResponseVM } from '../common/chatViewModel.js';
let ChatAccessibilityProvider = class ChatAccessibilityProvider {
    constructor(_accessibleViewService) {
        this._accessibleViewService = _accessibleViewService;
    }
    getWidgetRole() {
        return 'list';
    }
    getRole(element) {
        return 'listitem';
    }
    getWidgetAriaLabel() {
        return ( localize(7374, "Chat"));
    }
    getAriaLabel(element) {
        if (isRequestVM(element)) {
            return element.messageText;
        }
        if (isResponseVM(element)) {
            return this._getLabelWithCodeBlockCount(element);
        }
        return '';
    }
    _getLabelWithCodeBlockCount(element) {
        const accessibleViewHint = this._accessibleViewService.getOpenAriaHint("accessibility.verbosity.panelChat" );
        let label = '';
        const fileTreeCount = element.response.value.filter((v) => !('value' in v))?.length ?? 0;
        let fileTreeCountHint = '';
        switch (fileTreeCount) {
            case 0:
                break;
            case 1:
                fileTreeCountHint = ( localize(7375, "1 file tree"));
                break;
            default:
                fileTreeCountHint = ( localize(7376, "{0} file trees", fileTreeCount));
                break;
        }
        const codeBlockCount = marked.lexer(( (element.response.toString()))).filter(token => token.type === 'code')?.length ?? 0;
        switch (codeBlockCount) {
            case 0:
                label = accessibleViewHint ? ( localize(
                    7377,
                    "{0} {1} {2}",
                    fileTreeCountHint,
                     (element.response.toString()),
                    accessibleViewHint
                )) : ( localize(
                    7378,
                    "{0} {1}",
                    fileTreeCountHint,
                     (element.response.toString())
                ));
                break;
            case 1:
                label = accessibleViewHint ? ( localize(
                    7379,
                    "{0} 1 code block: {1} {2}",
                    fileTreeCountHint,
                     (element.response.toString()),
                    accessibleViewHint
                )) : ( localize(
                    7380,
                    "{0} 1 code block: {1}",
                    fileTreeCountHint,
                     (element.response.toString())
                ));
                break;
            default:
                label = accessibleViewHint ? ( localize(
                    7381,
                    "{0} {1} code blocks: {2}",
                    fileTreeCountHint,
                    codeBlockCount,
                     (element.response.toString()),
                    accessibleViewHint
                )) : ( localize(
                    7382,
                    "{0} {1} code blocks",
                    fileTreeCountHint,
                    codeBlockCount,
                     (element.response.toString())
                ));
                break;
        }
        return label;
    }
};
ChatAccessibilityProvider = ( (__decorate([
    ( (__param(0, IAccessibleViewService)))
], ChatAccessibilityProvider)));
export { ChatAccessibilityProvider };
