import { __decorate, __param } from '../../../../../../../../external/tslib/tslib.es6.js';
import { $, clearNode } from '../../../../../base/browser/dom.js';
import { Codicon } from '../../../../../base/common/codicons.js';
import { Emitter } from '../../../../../base/common/event.js';
import { Disposable, DisposableStore } from '../../../../../base/common/lifecycle.js';
import { localize } from '../../../../../nls.js';
import { IInstantiationService } from '../../../../../platform/instantiation/common/instantiation.js';
import { ChatConfirmationWidget } from './chatConfirmationWidget.js';
import { ChatProgressContentPart } from './chatProgressContentPart.js';
let ChatToolInvocationPart = class ChatToolInvocationPart extends Disposable {
    constructor(toolInvocation, context, renderer, instantiationService) {
        super();
        this._onDidChangeHeight = this._register(( (new Emitter())));
        this.onDidChangeHeight = this._onDidChangeHeight.event;
        this.domNode = $('.chat-tool-invocation-part');
        const partStore = this._register(( (new DisposableStore())));
        const render = () => {
            clearNode(this.domNode);
            const subPart = partStore.add(instantiationService.createInstance(ChatToolInvocationSubPart, toolInvocation, context, renderer));
            this.domNode.appendChild(subPart.domNode);
            partStore.add(subPart.onNeedsRerender(() => {
                render();
                this._onDidChangeHeight.fire();
            }));
        };
        render();
    }
    hasSameContent(other, followingContent, element) {
        return other.kind === 'toolInvocation' || other.kind === 'toolInvocationSerialized';
    }
    addDisposable(disposable) {
        this._register(disposable);
    }
};
ChatToolInvocationPart = ( (__decorate([
    ( (__param(3, IInstantiationService)))
], ChatToolInvocationPart)));
let ChatToolInvocationSubPart = class ChatToolInvocationSubPart extends Disposable {
    constructor(toolInvocation, context, renderer, instantiationService) {
        super();
        this._onNeedsRerender = this._register(( (new Emitter())));
        this.onNeedsRerender = this._onNeedsRerender.event;
        if (toolInvocation.kind === 'toolInvocation' && toolInvocation.confirmationMessages) {
            const title = toolInvocation.confirmationMessages.title;
            const message = toolInvocation.confirmationMessages.message;
            const confirmWidget = this._register(instantiationService.createInstance(ChatConfirmationWidget, title, message, [{ label: ( localize(10732, "Continue")), data: true }, { label: ( localize(10733, "Cancel")), data: false, isSecondary: true }]));
            this.domNode = confirmWidget.domNode;
            this._register(confirmWidget.onDidClick(button => {
                toolInvocation.confirmed.complete(button.data);
            }));
            toolInvocation.confirmed.p.then(() => this._onNeedsRerender.fire());
        }
        else {
            const message = toolInvocation.invocationMessage + '…';
            const progressMessage = {
                kind: 'progressMessage',
                content: { value: message }
            };
            const iconOverride = toolInvocation.isConfirmed === false ? Codicon.error : undefined;
            const progressPart = this._register(instantiationService.createInstance(ChatProgressContentPart, progressMessage, renderer, context, undefined, true, iconOverride));
            this.domNode = progressPart.domNode;
        }
    }
};
ChatToolInvocationSubPart = ( (__decorate([
    ( (__param(3, IInstantiationService)))
], ChatToolInvocationSubPart)));
export { ChatToolInvocationPart };
