import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { Codicon } from '../../../../base/common/codicons.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { IClipboardService } from '../../../../platform/clipboard/common/clipboardService.service.js';
import { localize } from '../../../../nls.js';
import { IConfigurationService } from '../../../../platform/configuration/common/configuration.service.js';
let ChatImageDropAndPaste = class ChatImageDropAndPaste extends Disposable {
    constructor(inputPart, clipboardService, configurationService) {
        super();
        this.inputPart = inputPart;
        this.clipboardService = clipboardService;
        this.configurationService = configurationService;
        this._register(this.inputPart.inputEditor.onDidPaste((e) => {
            if (this.configurationService.getValue('chat.experimental.imageAttachments')) {
                this._handlePaste();
            }
        }));
    }
    async _handlePaste() {
        const currClipboard = await this.clipboardService.readImage();
        if (!currClipboard || !isImage(currClipboard)) {
            return;
        }
        const context = await getImageAttachContext(currClipboard);
        if (!context) {
            return;
        }
        const currentContextIds = ( (new Set(
             (Array.from(this.inputPart.attachedContext).map(context => context.id))
        )));
        const filteredContext = [];
        if (!( (currentContextIds.has(context.id)))) {
            currentContextIds.add(context.id);
            filteredContext.push(context);
        }
        this.inputPart.attachContext(false, ...filteredContext);
    }
};
ChatImageDropAndPaste = ( (__decorate([
    ( (__param(1, IClipboardService))),
    ( (__param(2, IConfigurationService)))
], ChatImageDropAndPaste)));
async function getImageAttachContext(data) {
    return {
        value: data,
        id: await imageToHash(data),
        name: ( localize(7386, 'Pasted Image')),
        isImage: true,
        icon: Codicon.fileMedia,
        isDynamic: true,
    };
}
async function imageToHash(data) {
    const hashBuffer = await crypto.subtle.digest('SHA-256', data);
    const hashArray = Array.from(( (new Uint8Array(hashBuffer))));
    return ( (hashArray.map(b => ( (b.toString(16))).padStart(2, '0')))).join('');
}
function isImage(array) {
    if (array.length < 4) {
        return false;
    }
    const identifier = {
        png: [0x89, 0x50, 0x4E, 0x47, 0x0D, 0x0A, 0x1A, 0x0A],
        jpeg: [0xFF, 0xD8, 0xFF],
        bmp: [0x42, 0x4D],
        gif: [0x47, 0x49, 0x46, 0x38],
        tiff: [0x49, 0x49, 0x2A, 0x00]
    };
    return (
         (( (Object.values(identifier))).some((signature) => signature.every((byte, index) => array[index] === byte)))
    );
}
export { ChatImageDropAndPaste, imageToHash, isImage };
