import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { addDisposableListener, $, getWindow, EventHelper, EventType } from '../../../../base/browser/dom.js';
import { StandardMouseEvent } from '../../../../base/browser/mouseEvent.js';
import { getDefaultHoverDelegate } from '../../../../base/browser/ui/hover/hoverDelegateFactory.js';
import { Lazy } from '../../../../base/common/lazy.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { generateUuid } from '../../../../base/common/uuid.js';
import { ICodeEditorService } from '../../../../editor/browser/services/codeEditorService.js';
import { EditorContextKeys } from '../../../../editor/common/editorContextKeys.js';
import { SymbolKinds } from '../../../../editor/common/languages.js';
import { ILanguageService } from '../../../../editor/common/languages/language.js';
import { getIconClasses } from '../../../../editor/common/services/getIconClasses.js';
import { ILanguageFeaturesService } from '../../../../editor/common/services/languageFeatures.js';
import { IModelService } from '../../../../editor/common/services/model.js';
import { DefinitionAction } from '../../../../editor/contrib/gotoSymbol/browser/goToCommands.js';
import { localize2, localize } from '../../../../nls.js';
import { createAndFillInContextMenuActions } from '../../../../platform/actions/browser/menuEntryActionViewItem.js';
import { registerAction2, Action2, MenuId } from '../../../../platform/actions/common/actions.js';
import { IMenuService } from '../../../../platform/actions/common/actions.service.js';
import { ICommandService } from '../../../../platform/commands/common/commands.service.js';
import { IContextKeyService } from '../../../../platform/contextkey/common/contextkey.service.js';
import { IContextMenuService } from '../../../../platform/contextview/browser/contextView.service.js';
import { FileKind } from '../../../../platform/files/common/files.js';
import { IFileService } from '../../../../platform/files/common/files.service.js';
import { IHoverService } from '../../../../platform/hover/browser/hover.service.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { ILabelService } from '../../../../platform/label/common/label.service.js';
import { ITelemetryService } from '../../../../platform/telemetry/common/telemetry.service.js';
import { fillEditorsDragData } from '../../../browser/dnd.js';
import { ResourceContextKey } from '../../../common/contextkeys.js';
import { ExplorerFolderContext } from '../../files/common/files.js';
import { IChatVariablesService } from '../common/chatVariables.service.js';
import { IChatWidgetService } from './chat.service.js';
let InlineAnchorWidget = class InlineAnchorWidget extends Disposable {
    constructor(element, data, originalContextKeyService, contextMenuService, fileService, hoverService, instantiationService, labelService, languageFeaturesService, languageService, menuService, modelService, telemetryService) {
        super();
        const contextKeyService = this._register(originalContextKeyService.createScoped(element));
        const anchorId = ( (new Lazy(generateUuid)));
        element.classList.add('chat-inline-anchor-widget', 'show-file-icons');
        let iconText;
        let iconClasses;
        let location;
        let contextMenuId;
        let contextMenuArg;
        if (data.kind === 'symbol') {
            location = data.symbol.location;
            contextMenuId = MenuId.ChatInlineSymbolAnchorContext;
            contextMenuArg = location;
            iconText = data.symbol.name;
            iconClasses = ['codicon', ...getIconClasses(modelService, languageService, undefined, undefined, SymbolKinds.toIcon(data.symbol.kind))];
            const model = modelService.getModel(location.uri);
            if (model) {
                const hasDefinitionProvider = EditorContextKeys.hasDefinitionProvider.bindTo(contextKeyService);
                const hasReferenceProvider = EditorContextKeys.hasReferenceProvider.bindTo(contextKeyService);
                const updateContents = () => {
                    if (model.isDisposed()) {
                        return;
                    }
                    hasDefinitionProvider.set(( (languageFeaturesService.definitionProvider.has(model))));
                    hasReferenceProvider.set(( (languageFeaturesService.definitionProvider.has(model))));
                };
                updateContents();
                this._register(languageFeaturesService.definitionProvider.onDidChange(updateContents));
                this._register(languageFeaturesService.referenceProvider.onDidChange(updateContents));
            }
            this._register(addDisposableListener(element, 'click', () => {
                telemetryService.publicLog2('chat.inlineAnchor.openSymbol', {
                    anchorId: anchorId.value
                });
            }));
        }
        else {
            location = data;
            contextMenuId = MenuId.ChatInlineResourceAnchorContext;
            contextMenuArg = location.uri;
            const resourceContextKey = this._register(( (new ResourceContextKey(contextKeyService, fileService, languageService, modelService))));
            resourceContextKey.set(location.uri);
            const label = labelService.getUriBasenameLabel(location.uri);
            iconText = location.range && data.kind !== 'symbol' ?
                `${label}#${location.range.startLineNumber}-${location.range.endLineNumber}` :
                label;
            const fileKind = location.uri.path.endsWith('/') ? FileKind.FOLDER : FileKind.FILE;
            iconClasses = getIconClasses(modelService, languageService, location.uri, fileKind);
            const isFolderContext = ExplorerFolderContext.bindTo(contextKeyService);
            fileService.stat(location.uri)
                .then(stat => {
                isFolderContext.set(stat.isDirectory);
            })
                .catch(() => { });
            this._register(addDisposableListener(element, 'click', () => {
                telemetryService.publicLog2('chat.inlineAnchor.openResource', {
                    anchorId: anchorId.value
                });
            }));
        }
        const iconEl = $('span.icon');
        iconEl.classList.add(...iconClasses);
        element.replaceChildren(iconEl, $('span.icon-label', {}, iconText));
        const fragment = location.range ? `${location.range.startLineNumber}-${location.range.endLineNumber}` : '';
        element.setAttribute('data-href', ( (location.uri.with({ fragment }).toString())));
        this._register(addDisposableListener(element, EventType.CONTEXT_MENU, domEvent => {
            const event = ( (new StandardMouseEvent(getWindow(domEvent), domEvent)));
            EventHelper.stop(domEvent, true);
            contextMenuService.showContextMenu({
                contextKeyService,
                getAnchor: () => event,
                getActions: () => {
                    const menu = menuService.getMenuActions(contextMenuId, contextKeyService, { arg: contextMenuArg });
                    const primary = [];
                    createAndFillInContextMenuActions(menu, primary);
                    return primary;
                },
            });
        }));
        const relativeLabel = labelService.getUriLabel(location.uri, { relative: true });
        this._register(hoverService.setupManagedHover(getDefaultHoverDelegate('element'), element, relativeLabel));
        element.draggable = true;
        this._register(addDisposableListener(element, 'dragstart', e => {
            instantiationService.invokeFunction(accessor => fillEditorsDragData(accessor, [location.uri], e));
            e.dataTransfer?.setDragImage(element, 0, 0);
        }));
    }
};
InlineAnchorWidget = ( (__decorate([
    ( (__param(2, IContextKeyService))),
    ( (__param(3, IContextMenuService))),
    ( (__param(4, IFileService))),
    ( (__param(5, IHoverService))),
    ( (__param(6, IInstantiationService))),
    ( (__param(7, ILabelService))),
    ( (__param(8, ILanguageFeaturesService))),
    ( (__param(9, ILanguageService))),
    ( (__param(10, IMenuService))),
    ( (__param(11, IModelService))),
    ( (__param(12, ITelemetryService)))
], InlineAnchorWidget)));
registerAction2(class AddFileToChatAction extends Action2 {
    static { this.id = 'chat.inlineResourceAnchor.addFileToChat'; }
    constructor() {
        super({
            id: AddFileToChatAction.id,
            title: {
                ...( localize2(10758, "Add File to Chat")),
            },
            menu: [{
                    id: MenuId.ChatInlineResourceAnchorContext,
                    group: 'chat',
                    order: 1,
                    when: ( (ExplorerFolderContext.negate())),
                }]
        });
    }
    async run(accessor, resource) {
        const chatWidgetService = accessor.get(IChatWidgetService);
        const variablesService = accessor.get(IChatVariablesService);
        const widget = chatWidgetService.lastFocusedWidget;
        if (!widget) {
            return;
        }
        variablesService.attachContext('file', resource, widget.location);
    }
});
registerAction2(class GoToDefinitionAction extends Action2 {
    static { this.id = 'chat.inlineSymbolAnchor.goToDefinition'; }
    constructor() {
        super({
            id: GoToDefinitionAction.id,
            title: {
                ...( localize2(10759, "Go to Definition")),
                mnemonicTitle: ( localize(10760, "Go to &&Definition")),
            },
            precondition: EditorContextKeys.hasDefinitionProvider,
            menu: [{
                    id: MenuId.ChatInlineSymbolAnchorContext,
                    group: 'navigation',
                    order: 1.1,
                },]
        });
    }
    async run(accessor, location) {
        const editorService = accessor.get(ICodeEditorService);
        await editorService.openCodeEditor({
            resource: location.uri, options: {
                selection: {
                    startColumn: location.range.startColumn,
                    startLineNumber: location.range.startLineNumber,
                }
            }
        }, null);
        const action = ( (new DefinitionAction(
            { openToSide: false, openInPeek: false, muteMessage: true },
            { title: { value: '', original: '' }, id: '', precondition: undefined }
        )));
        return action.run(accessor);
    }
});
registerAction2(class GoToReferencesAction extends Action2 {
    static { this.id = 'chat.inlineSymbolAnchor.goToReferences'; }
    constructor() {
        super({
            id: GoToReferencesAction.id,
            title: {
                ...( localize2(10761, "Go to References")),
                mnemonicTitle: ( localize(10762, "Go to &&References")),
            },
            precondition: EditorContextKeys.hasReferenceProvider,
            menu: [{
                    id: MenuId.ChatInlineSymbolAnchorContext,
                    group: 'navigation',
                    order: 1.1,
                },]
        });
    }
    async run(accessor, location) {
        const editorService = accessor.get(ICodeEditorService);
        const commandService = accessor.get(ICommandService);
        await editorService.openCodeEditor({
            resource: location.uri, options: {
                selection: {
                    startColumn: location.range.startColumn,
                    startLineNumber: location.range.startLineNumber,
                }
            }
        }, null);
        await commandService.executeCommand('editor.action.goToReferences');
    }
});
export { InlineAnchorWidget };
