import { ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { ctxCommentEditorFocused } from './simpleCommentEditor.js';
import { CommentContextKeys } from '../common/commentContextKeys.js';
import { localize } from '../../../../nls.js';
import { ToggleTabFocusModeAction } from '../../../../editor/contrib/toggleTabFocusMode/browser/toggleTabFocusMode.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
var CommentAccessibilityHelpNLS;
( ((function(CommentAccessibilityHelpNLS) {
    CommentAccessibilityHelpNLS.intro = ( localize(
        5355,
        "The editor contains commentable range(s). Some useful commands include:"
    ));
    CommentAccessibilityHelpNLS.tabFocus = ( localize(
        5356,
        "This widget contains a text area, for composition of new comments, and actions, that can be tabbed to once tab moves focus mode has been enabled with the command Toggle Tab Key Moves Focus{0}.",
        `<keybinding:${ToggleTabFocusModeAction.ID}>`
    ));
    CommentAccessibilityHelpNLS.commentCommands = ( localize(5357, "Some useful comment commands include:"));
    CommentAccessibilityHelpNLS.escape = ( localize(5358, "- Dismiss Comment (Escape)"));
    CommentAccessibilityHelpNLS.nextRange = ( localize(
        5359,
        "- Go to Next Commenting Range{0}.",
        `<keybinding:${"editor.action.nextCommentingRange" }>`
    ));
    CommentAccessibilityHelpNLS.previousRange = ( localize(
        5360,
        "- Go to Previous Commenting Range{0}.",
        `<keybinding:${"editor.action.previousCommentingRange" }>`
    ));
    CommentAccessibilityHelpNLS.nextCommentThread = ( localize(
        5361,
        "- Go to Next Comment Thread{0}.",
        `<keybinding:${"editor.action.nextCommentThreadAction" }>`
    ));
    CommentAccessibilityHelpNLS.previousCommentThread = ( localize(
        5362,
        "- Go to Previous Comment Thread{0}.",
        `<keybinding:${"editor.action.previousCommentThreadAction" }>`
    ));
    CommentAccessibilityHelpNLS.nextCommentedRange = ( localize(
        5363,
        "- Go to Next Commented Range{0}.",
        `<keybinding:${"editor.action.nextCommentedRangeAction" }>`
    ));
    CommentAccessibilityHelpNLS.previousCommentedRange = ( localize(
        5364,
        "- Go to Previous Commented Range{0}.",
        `<keybinding:${"editor.action.previousCommentedRangeAction" }>`
    ));
    CommentAccessibilityHelpNLS.addComment = ( localize(
        5365,
        "- Add Comment on Current Selection{0}.",
        `<keybinding:${"workbench.action.addComment" }>`
    ));
    CommentAccessibilityHelpNLS.submitComment = ( localize(
        5366,
        "- Submit Comment{0}.",
        `<keybinding:${"editor.action.submitComment" }>`
    ));
})(CommentAccessibilityHelpNLS || (CommentAccessibilityHelpNLS = {}))));
class CommentsAccessibilityHelpProvider extends Disposable {
    constructor() {
        super(...arguments);
        this.id = "comments" ;
        this.verbositySettingKey = "accessibility.verbosity.comments" ;
        this.options = { type: "help"  };
    }
    provideContent() {
        return [CommentAccessibilityHelpNLS.tabFocus, CommentAccessibilityHelpNLS.commentCommands, CommentAccessibilityHelpNLS.escape, CommentAccessibilityHelpNLS.addComment, CommentAccessibilityHelpNLS.submitComment, CommentAccessibilityHelpNLS.nextRange, CommentAccessibilityHelpNLS.previousRange].join('\n');
    }
    onClose() {
        this._element?.focus();
    }
}
class CommentsAccessibilityHelp {
    constructor() {
        this.priority = 110;
        this.name = 'comments';
        this.type = "help" ;
        this.when = ( (ContextKeyExpr.or(ctxCommentEditorFocused, CommentContextKeys.commentFocused)));
    }
    getProvider(accessor) {
        return accessor.get(IInstantiationService).createInstance(CommentsAccessibilityHelpProvider);
    }
}
export { CommentAccessibilityHelpNLS, CommentsAccessibilityHelp, CommentsAccessibilityHelpProvider };
