import { localize } from '../../../../nls.js';
import { RawContextKey, ContextKeyExpr } from '../../../../platform/contextkey/common/contextkey.js';
import { TERMINAL_VIEW_ID } from './terminal.js';
var TerminalContextKeys;
( ((function(TerminalContextKeys) {
    TerminalContextKeys.isOpen = ( (new RawContextKey("terminalIsOpen" , false, true)));
    TerminalContextKeys.focus = ( (new RawContextKey(
        "terminalFocus" ,
        false,
        ( localize(8943, "Whether the terminal is focused."))
    )));
    TerminalContextKeys.focusInAny = ( (new RawContextKey(
        "terminalFocusInAny" ,
        false,
        ( localize(
            8944,
            "Whether any terminal is focused, including detached terminals used in other UI."
        ))
    )));
    TerminalContextKeys.editorFocus = ( (new RawContextKey(
        "terminalEditorFocus" ,
        false,
        ( localize(8945, "Whether a terminal in the editor area is focused."))
    )));
    TerminalContextKeys.count = ( (new RawContextKey("terminalCount" , 0, ( localize(8946, "The current number of terminals.")))));
    TerminalContextKeys.groupCount = ( (new RawContextKey("terminalGroupCount" , 0, true)));
    TerminalContextKeys.tabsNarrow = ( (new RawContextKey(
        "isTerminalTabsNarrow" ,
        false,
        true
    )));
    TerminalContextKeys.terminalHasFixedWidth = ( (new RawContextKey(
        "terminalHasFixedWidth" ,
        false,
        true
    )));
    TerminalContextKeys.tabsFocus = ( (new RawContextKey(
        "terminalTabsFocus" ,
        false,
        ( localize(8947, "Whether the terminal tabs widget is focused."))
    )));
    TerminalContextKeys.webExtensionContributedProfile = ( (new RawContextKey(
        "terminalWebExtensionContributedProfile" ,
        false,
        true
    )));
    TerminalContextKeys.terminalHasBeenCreated = ( (new RawContextKey(
        "terminalHasBeenCreated" ,
        false,
        true
    )));
    TerminalContextKeys.terminalEditorActive = ( (new RawContextKey(
        "terminalEditorActive" ,
        false,
        true
    )));
    TerminalContextKeys.tabsMouse = ( (new RawContextKey("terminalTabsMouse" , false, true)));
    TerminalContextKeys.shellType = ( (new RawContextKey(
        "terminalShellType" ,
        undefined,
        { type: 'string', description: ( localize(
            8948,
            "The shell type of the active terminal, this is set if the type can be detected."
        )) }
    )));
    TerminalContextKeys.altBufferActive = ( (new RawContextKey(
        "terminalAltBufferActive" ,
        false,
        ( localize(8949, "Whether the terminal's alt buffer is active."))
    )));
    TerminalContextKeys.suggestWidgetVisible = ( (new RawContextKey(
        "terminalSuggestWidgetVisible" ,
        false,
        ( localize(8950, "Whether the terminal's suggest widget is visible."))
    )));
    TerminalContextKeys.notFocus = ( (TerminalContextKeys.focus.toNegated()));
    TerminalContextKeys.viewShowing = ( (new RawContextKey(
        "terminalViewShowing" ,
        false,
        ( localize(8951, "Whether the terminal view is showing"))
    )));
    TerminalContextKeys.textSelected = ( (new RawContextKey(
        "terminalTextSelected" ,
        false,
        ( localize(8952, "Whether text is selected in the active terminal."))
    )));
    TerminalContextKeys.textSelectedInFocused = ( (new RawContextKey(
        "terminalTextSelectedInFocused" ,
        false,
        ( localize(8953, "Whether text is selected in a focused terminal."))
    )));
    TerminalContextKeys.notTextSelected = ( (TerminalContextKeys.textSelected.toNegated()));
    TerminalContextKeys.findVisible = ( (new RawContextKey(
        "terminalFindVisible" ,
        false,
        true
    )));
    TerminalContextKeys.notFindVisible = ( (TerminalContextKeys.findVisible.toNegated()));
    TerminalContextKeys.findInputFocus = ( (new RawContextKey(
        "terminalFindInputFocused" ,
        false,
        true
    )));
    TerminalContextKeys.findFocus = ( (new RawContextKey(
        "terminalFindFocused" ,
        false,
        true
    )));
    TerminalContextKeys.notFindFocus = ( (TerminalContextKeys.findInputFocus.toNegated()));
    TerminalContextKeys.processSupported = ( (new RawContextKey(
        "terminalProcessSupported" ,
        false,
        ( localize(
            8954,
            "Whether terminal processes can be launched in the current workspace."
        ))
    )));
    TerminalContextKeys.tabsSingularSelection = ( (new RawContextKey(
        "terminalTabsSingularSelection" ,
        false,
        ( localize(8955, "Whether one terminal is selected in the terminal tabs list."))
    )));
    TerminalContextKeys.splitTerminal = ( (new RawContextKey(
        "terminalSplitTerminal" ,
        false,
        ( localize(8956, "Whether the focused tab's terminal is a split terminal."))
    )));
    TerminalContextKeys.inTerminalRunCommandPicker = ( (new RawContextKey(
        "inTerminalRunCommandPicker" ,
        false,
        ( localize(8957, "Whether the terminal run command picker is currently open."))
    )));
    TerminalContextKeys.terminalShellIntegrationEnabled = ( (new RawContextKey(
        "terminalShellIntegrationEnabled" ,
        false,
        ( localize(8958, "Whether shell integration is enabled in the active terminal"))
    )));
    TerminalContextKeys.shouldShowViewInlineActions = ( (ContextKeyExpr.and(
         (ContextKeyExpr.equals('view', TERMINAL_VIEW_ID)),
         (ContextKeyExpr.notEquals(
            `config.${"terminal.integrated.tabs.hideCondition" }`,
            'never'
        )),
         (ContextKeyExpr.or(
            ContextKeyExpr.not(`config.${"terminal.integrated.tabs.enabled" }`),
             (ContextKeyExpr.and( (ContextKeyExpr.equals(
                `config.${"terminal.integrated.tabs.showActions" }`,
                'singleTerminal'
            )),  (ContextKeyExpr.equals("terminalGroupCount", 1)))),
             (ContextKeyExpr.and( (ContextKeyExpr.equals(
                `config.${"terminal.integrated.tabs.showActions" }`,
                'singleTerminalOrNarrow'
            )),  (ContextKeyExpr.or(
                 (ContextKeyExpr.equals("terminalGroupCount", 1)),
                 (ContextKeyExpr.has("isTerminalTabsNarrow" ))
            )))),
             (ContextKeyExpr.and( (ContextKeyExpr.equals(
                `config.${"terminal.integrated.tabs.showActions" }`,
                'singleGroup'
            )),  (ContextKeyExpr.equals("terminalGroupCount", 1)))),
             (ContextKeyExpr.equals(
                `config.${"terminal.integrated.tabs.showActions" }`,
                'always'
            ))
        ))
    )));
})(TerminalContextKeys || (TerminalContextKeys = {}))));
export { TerminalContextKeys };
