import { Color } from '../../../../base/common/color.js';
import { registerColor } from '../../../../platform/theme/common/colorUtils.js';
import { localize } from '../../../../nls.js';
import { Codicon } from '../../../../base/common/codicons.js';
class BaseBadge {
    constructor(descriptorFn, stylesFn) {
        this.descriptorFn = descriptorFn;
        this.stylesFn = stylesFn;
    }
    getDescription() {
        return this.descriptorFn(null);
    }
    getColors(theme) {
        return this.stylesFn?.(theme);
    }
}
class NumberBadge extends BaseBadge {
    constructor(number, descriptorFn) {
        super(descriptorFn, undefined);
        this.number = number;
        this.number = number;
    }
    getDescription() {
        return this.descriptorFn(this.number);
    }
}
class IconBadge extends BaseBadge {
    constructor(icon, descriptorFn, stylesFn) {
        super(descriptorFn, stylesFn);
        this.icon = icon;
    }
}
class ProgressBadge extends BaseBadge {
    constructor(descriptorFn) {
        super(descriptorFn, undefined);
    }
}
class WarningBadge extends IconBadge {
    constructor(descriptorFn) {
        super(Codicon.warning, descriptorFn, (theme) => ({
            badgeBackground: theme.getColor(activityWarningBadgeBackground),
            badgeForeground: theme.getColor(activityWarningBadgeForeground),
            badgeBorder: undefined,
        }));
    }
}
const activityWarningBadgeForeground = registerColor('activityWarningBadge.foreground', { dark: ( (Color.black.lighten(0.2))), light: Color.white, hcDark: null, hcLight: ( (Color.black.lighten(0.2))) }, ( localize(5316, 'Foreground color of the warning activity badge')));
const activityWarningBadgeBackground = registerColor('activityWarningBadge.background', { dark: '#CCA700', light: '#BF8803', hcDark: null, hcLight: '#CCA700' }, ( localize(5317, 'Background color of the warning activity badge')));
registerColor('activityErrorBadge.foreground', { dark: ( (Color.black.lighten(0.2))), light: Color.white, hcDark: null, hcLight: ( (Color.black.lighten(0.2))) }, ( localize(5318, 'Foreground color of the error activity badge')));
registerColor('activityErrorBadge.background', { dark: '#F14C4C', light: '#E51400', hcDark: null, hcLight: '#F14C4C' }, ( localize(5319, 'Background color of the error activity badge')));
export { IconBadge, NumberBadge, ProgressBadge, WarningBadge };
